@@ -40,6 +40,7 @@
 #endif
 
 namespace jxl {
+namespace extras {
 
 #if JPEGXL_ENABLE_JPEG
 namespace {
@@ -85,8 +86,8 @@ Status ReadICCProfile(jpeg_decompress_struct* const cinfo,
        marker = marker->next) {
     // marker is initialized by libjpeg, which we are not instrumenting with
     // msan.
-    UnpoisonMemory(marker, sizeof(*marker));
-    UnpoisonMemory(marker->data, marker->data_length);
+    msan::UnpoisonMemory(marker, sizeof(*marker));
+    msan::UnpoisonMemory(marker->data, marker->data_length);
     if (!MarkerIsICC(marker)) continue;
 
     const int current_marker = marker->data[kICCSignatureSize];
@@ -154,8 +155,8 @@ void ReadExif(jpeg_decompress_struct* const cinfo, PaddedBytes* const exif) {
        marker = marker->next) {
     // marker is initialized by libjpeg, which we are not instrumenting with
     // msan.
-    UnpoisonMemory(marker, sizeof(*marker));
-    UnpoisonMemory(marker->data, marker->data_length);
+    msan::UnpoisonMemory(marker, sizeof(*marker));
+    msan::UnpoisonMemory(marker->data, marker->data_length);
     if (!MarkerIsExif(marker)) continue;
     size_t marker_length = marker->data_length - kExifSignatureSize;
     exif->resize(marker_length);
@@ -235,17 +236,22 @@ void MyOutputMessage(j_common_ptr cinfo) {
 }  // namespace
 #endif  // JPEGXL_ENABLE_JPEG
 
-Status DecodeImageJPG(const Span<const uint8_t> bytes, ThreadPool* pool,
+Status DecodeImageJPGCoefficients(Span<const uint8_t> bytes, CodecInOut* io) {
+  if (!IsJPG(bytes)) return false;
+  // Use brunsli JPEG decoder to read quantized coefficients.
+  if (!jpeg::DecodeImageJPG(bytes, io)) {
+    fprintf(stderr, "Corrupt or CMYK JPEG.\n");
+    return false;
+  }
+  return true;
+}
+
+Status DecodeImageJPG(const Span<const uint8_t> bytes,
+                      const ColorHints& color_hints, ThreadPool* pool,
                       CodecInOut* io, double* const elapsed_deinterleave) {
   if (elapsed_deinterleave != nullptr) *elapsed_deinterleave = 0;
   // Don't do anything for non-JPEG files (no need to report an error)
   if (!IsJPG(bytes)) return false;
-  const DecodeTarget target = io->dec_target;
-
-  // Use brunsli JPEG decoder to read quantized coefficients.
-  if (target == DecodeTarget::kQuantizedCoeffs) {
-    return jxl::jpeg::DecodeImageJPG(bytes, io);
-  }
 
 #if JPEGXL_ENABLE_JPEG
   // TODO(veluca): use JPEGData also for pixels?
@@ -260,11 +266,9 @@ Status DecodeImageJPG(const Span<const uint8_t> bytes, ThreadPool* pool,
 
   const auto try_catch_block = [&]() -> bool {
     jpeg_decompress_struct cinfo;
-#ifdef MEMORY_SANITIZER
     // cinfo is initialized by libjpeg, which we are not instrumenting with
     // msan, therefore we need to initialize cinfo here.
-    memset(&cinfo, 0, sizeof(cinfo));
-#endif
+    msan::UnpoisonMemory(&cinfo, sizeof(cinfo));
     // Setup error handling in jpeg library so we can deal with broken jpegs in
     // the fuzzer.
     jpeg_error_mgr jerr;
@@ -310,11 +314,9 @@ Status DecodeImageJPG(const Span<const uint8_t> bytes, ThreadPool* pool,
     if (nbcomp != 1 && nbcomp != 3) {
       return failure("unsupported number of components in JPEG");
     }
-    (void)io->dec_hints.Foreach(
-        [](const std::string& key, const std::string& /*value*/) {
-          JXL_WARNING("JPEG decoder ignoring %s hint", key.c_str());
-          return true;
-        });
+    if (!ApplyColorHints(color_hints, /*color_already_set=*/true, false, io)) {
+      return failure("ApplyColorHints failed");
+    }
 
     jpeg_start_decompress(&cinfo);
     JXL_ASSERT(cinfo.output_components == nbcomp);
@@ -323,8 +325,9 @@ Status DecodeImageJPG(const Span<const uint8_t> bytes, ThreadPool* pool,
     for (size_t y = 0; y < image.ysize(); ++y) {
       JSAMPROW rows[] = {row.get()};
       jpeg_read_scanlines(&cinfo, rows, 1);
-      UnpoisonMemory(row.get(), sizeof(JSAMPLE) * cinfo.output_components *
-                                    cinfo.image_width);
+      msan::UnpoisonMemory(
+          row.get(),
+          sizeof(JSAMPLE) * cinfo.output_components * cinfo.image_width);
       auto start = Now();
       float* const JXL_RESTRICT output_row[] = {
           image.PlaneRow(0, y), image.PlaneRow(1, y), image.PlaneRow(2, y)};
@@ -367,7 +370,7 @@ Status EncodeWithLibJpeg(const ImageBundle* ib, const CodecInOut* io,
   jpeg_compress_struct cinfo;
   // cinfo is initialized by libjpeg, which we are not instrumenting with
   // msan.
-  UnpoisonMemory(&cinfo, sizeof(cinfo));
+  msan::UnpoisonMemory(&cinfo, sizeof(cinfo));
   jpeg_error_mgr jerr;
   cinfo.err = jpeg_std_error(&jerr);
   jpeg_create_compress(&cinfo);
@@ -420,7 +423,7 @@ Status EncodeWithLibJpeg(const ImageBundle* ib, const CodecInOut* io,
   bytes->resize(size);
   // Compressed image data is initialized by libjpeg, which we are not
   // instrumenting with msan.
-  UnpoisonMemory(buffer, size);
+  msan::UnpoisonMemory(buffer, size);
   std::copy_n(buffer, size, bytes->data());
   std::free(buffer);
   return true;
@@ -470,10 +473,17 @@ Status EncodeWithSJpeg(const ImageBundle* ib, size_t quality,
 }
 #endif  // JPEGXL_ENABLE_JPEG
 
+Status EncodeImageJPGCoefficients(const CodecInOut* io, PaddedBytes* bytes) {
+  auto write = [&bytes](const uint8_t* buf, size_t len) {
+    bytes->append(buf, buf + len);
+    return len;
+  };
+  return jpeg::WriteJpeg(*io->Main().jpeg_data, write);
+}
+
 Status EncodeImageJPG(const CodecInOut* io, JpegEncoder encoder, size_t quality,
                       YCbCrChromaSubsampling chroma_subsampling,
-                      ThreadPool* pool, PaddedBytes* bytes,
-                      const DecodeTarget target) {
+                      ThreadPool* pool, PaddedBytes* bytes) {
   if (io->Main().HasAlpha()) {
     return JXL_FAILURE("alpha is not supported");
   }
@@ -481,14 +491,6 @@ Status EncodeImageJPG(const CodecInOut* io, JpegEncoder encoder, size_t quality,
     return JXL_FAILURE("please specify a 0-100 JPEG quality");
   }
 
-  if (target == DecodeTarget::kQuantizedCoeffs) {
-    auto write = [&bytes](const uint8_t* buf, size_t len) {
-      bytes->append(buf, buf + len);
-      return len;
-    };
-    return jpeg::WriteJpeg(*io->Main().jpeg_data, write);
-  }
-
 #if JPEGXL_ENABLE_JPEG
   const ImageBundle* ib;
   ImageMetadata metadata = io->metadata.m;
@@ -515,4 +517,5 @@ Status EncodeImageJPG(const CodecInOut* io, JpegEncoder encoder, size_t quality,
 #endif  // JPEGXL_ENABLE_JPEG
 }
 
+}  // namespace extras
 }  // namespace jxl
