@@ -52,9 +52,9 @@ void DoUndoXYBInPlace(Image3F* idct, const Rect& rect, Op op,
     float* JXL_RESTRICT row2 = rect.PlaneRow(idct, 2, y);
     // All calculations are lane-wise, still some might require value-dependent
     // behaviour (e.g. NearestInt). Temporary unposion last vector tail.
-    UnpoisonMemory(row0 + xsize, sizeof(float) * (xsize_v - xsize));
-    UnpoisonMemory(row1 + xsize, sizeof(float) * (xsize_v - xsize));
-    UnpoisonMemory(row2 + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::UnpoisonMemory(row0 + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::UnpoisonMemory(row1 + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::UnpoisonMemory(row2 + xsize, sizeof(float) * (xsize_v - xsize));
     for (size_t x = 0; x < rect.xsize(); x += Lanes(d)) {
       const auto in_opsin_x = Load(d, row0 + x);
       const auto in_opsin_y = Load(d, row1 + x);
@@ -70,9 +70,9 @@ void DoUndoXYBInPlace(Image3F* idct, const Rect& rect, Op op,
       Store(op.Transform(d, linear_g), d, row1 + x);
       Store(op.Transform(d, linear_b), d, row2 + x);
     }
-    PoisonMemory(row0 + xsize, sizeof(float) * (xsize_v - xsize));
-    PoisonMemory(row1 + xsize, sizeof(float) * (xsize_v - xsize));
-    PoisonMemory(row2 + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::PoisonMemory(row0 + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::PoisonMemory(row1 + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::PoisonMemory(row2 + xsize, sizeof(float) * (xsize_v - xsize));
   }
 }
 
@@ -231,11 +231,11 @@ void FloatToRGBA8(const Image3F& input, const Rect& input_rect, bool is_rgba,
     // behaviour (e.g. NearestInt). Temporary unposion last vector tail.
     size_t xsize = output_buf_rect.xsize();
     size_t xsize_v = RoundUpTo(xsize, Lanes(d));
-    UnpoisonMemory(row_in_r + xsize, sizeof(float) * (xsize_v - xsize));
-    UnpoisonMemory(row_in_g + xsize, sizeof(float) * (xsize_v - xsize));
-    UnpoisonMemory(row_in_b + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::UnpoisonMemory(row_in_r + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::UnpoisonMemory(row_in_g + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::UnpoisonMemory(row_in_b + xsize, sizeof(float) * (xsize_v - xsize));
     if (row_in_a)
-      UnpoisonMemory(row_in_a + xsize, sizeof(float) * (xsize_v - xsize));
+      msan::UnpoisonMemory(row_in_a + xsize, sizeof(float) * (xsize_v - xsize));
     for (size_t x = 0; x < xsize; x += Lanes(d)) {
       auto rf = Clamp(zero, Load(d, row_in_r + x), one) * mul;
       auto gf = Clamp(zero, Load(d, row_in_g + x), one) * mul;
@@ -255,28 +255,56 @@ void FloatToRGBA8(const Image3F& input, const Rect& input_rect, bool is_rgba,
                   output_buf + base_ptr + bytes * x);
       }
     }
-    PoisonMemory(row_in_r + xsize, sizeof(float) * (xsize_v - xsize));
-    PoisonMemory(row_in_g + xsize, sizeof(float) * (xsize_v - xsize));
-    PoisonMemory(row_in_b + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::PoisonMemory(row_in_r + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::PoisonMemory(row_in_g + xsize, sizeof(float) * (xsize_v - xsize));
+    msan::PoisonMemory(row_in_b + xsize, sizeof(float) * (xsize_v - xsize));
     if (row_in_a)
-      PoisonMemory(row_in_a + xsize, sizeof(float) * (xsize_v - xsize));
+      msan::PoisonMemory(row_in_a + xsize, sizeof(float) * (xsize_v - xsize));
   }
 }
 
+// Upsample in horizonal (if hs=1) and vertical (if vs=1) the plane_in image
+// to the output plane_out image.
+// The output region "rect" in plane_out and a border around it of lf.Padding()
+// will be generated, as long as those pixels fall inside the image frame.
+// Otherwise the border pixels that fall outside the image frame in plane_out
+// are undefined.
+// "rect" is an area inside the plane_out image which corresponds to the
+// "frame_rect" area in the frame. plane_in and plane_out both are expected to
+// have a padding of kGroupDataXBorder and kGroupDataYBorder on either side of
+// X and Y coordinates. This means that when upsampling vertically the plane_out
+// row `kGroupDataXBorder + N` will be generated from the plane_in row
+// `kGroupDataXBorder + N / 2` (and a previous or next row).
 void DoYCbCrUpsampling(size_t hs, size_t vs, ImageF* plane_in, const Rect& rect,
                        const Rect& frame_rect, const FrameDimensions& frame_dim,
                        ImageF* plane_out, const LoopFilter& lf, ImageF* temp) {
+  JXL_DASSERT(SameSize(rect, frame_rect));
+  JXL_DASSERT(hs <= 1 && vs <= 1);
   // The pixel in (xoff, yoff) is the origin of the downsampling coordinate
   // system.
   size_t xoff = PassesDecoderState::kGroupDataXBorder;
   size_t yoff = PassesDecoderState::kGroupDataYBorder;
-  // This may over-copy, but it should always be safe to do so.
-  size_t y0 = rect.y0() - lf.Padding();
-  size_t y1 = rect.y0() + rect.ysize() + lf.Padding();
-  size_t x0 = rect.x0() - lf.Padding();
-  size_t x1 = rect.x0() + rect.xsize() + lf.Padding();
+
+  // This X,Y range is the intersection between the requested "rect" expanded
+  // with a lf.Padding() all around and the image frame translated to the
+  // coordinate system used by plane_out.
+  // All the pixels in the [x0, x1) x [y0, y1) range must be defined in the
+  // plane_out output at the end.
+  const size_t y0 = rect.y0() - std::min<size_t>(lf.Padding(), frame_rect.y0());
+  const size_t y1 = rect.y0() +
+                    std::min(frame_rect.y0() + rect.ysize() + lf.Padding(),
+                             frame_dim.ysize_padded) -
+                    frame_rect.y0();
+
+  const size_t x0 = rect.x0() - std::min<size_t>(lf.Padding(), frame_rect.x0());
+  const size_t x1 = rect.x0() +
+                    std::min(frame_rect.x0() + rect.xsize() + lf.Padding(),
+                             frame_dim.xsize_padded) -
+                    frame_rect.x0();
+
   if (hs == 0 && vs == 0) {
     Rect r(x0, y0, x1 - x0, y1 - y0);
+    JXL_CHECK_IMAGE_INITIALIZED(*plane_in, r);
     CopyImageTo(r, *plane_in, r, plane_out);
     return;
   }
@@ -288,8 +316,8 @@ void DoYCbCrUpsampling(size_t hs, size_t vs, ImageF* plane_in, const Rect& rect,
       plane_in->Row(y)[rect.x0() - 1] = plane_in->Row(y)[rect.x0()];
     }
   }
-  if (frame_rect.x0() + frame_rect.xsize() >= frame_dim.xsize_padded) {
-    size_t borderx = ((rect.x0() + rect.xsize() - xoff) >> hs) + xoff;
+  if (frame_rect.x0() + x1 - rect.x0() >= frame_dim.xsize_padded) {
+    ssize_t borderx = static_cast<ssize_t>(x1 - xoff + hs) / (1 << hs) + xoff;
     for (size_t y = 0; y < plane_in->ysize(); y++) {
       plane_in->Row(y)[borderx] = plane_in->Row(y)[borderx - 1];
     }
@@ -298,10 +326,8 @@ void DoYCbCrUpsampling(size_t hs, size_t vs, ImageF* plane_in, const Rect& rect,
     memcpy(plane_in->Row(rect.y0() - 1), plane_in->Row(rect.y0()),
            plane_in->xsize() * sizeof(float));
   }
-  if (frame_rect.y0() + frame_rect.ysize() >= frame_dim.ysize_padded) {
-    ssize_t bordery =
-        static_cast<ssize_t>(rect.y0() + rect.ysize() - yoff) / (1 << vs) +
-        yoff;
+  if (frame_rect.y0() + y1 - rect.y0() >= frame_dim.ysize_padded) {
+    ssize_t bordery = static_cast<ssize_t>(y1 - yoff + vs) / (1 << vs) + yoff;
     memcpy(plane_in->Row(bordery), plane_in->Row(bordery - 1),
            plane_in->xsize() * sizeof(float));
   }
@@ -371,6 +397,10 @@ void DoYCbCrUpsampling(size_t hs, size_t vs, ImageF* plane_in, const Rect& rect,
   } else {
     CopyImageTo(*temp, plane_out);
   }
+
+  // The output must be initialized including the lf.Padding() around the image
+  // for all the pixels that fall inside the image frame.
+  JXL_CHECK_IMAGE_INITIALIZED(*plane_out, Rect(x0, y0, x1 - x0, y1 - y0));
 }
 
 // NOLINTNEXTLINE(google-readability-namespace-comments)
@@ -400,6 +430,7 @@ Rect ScaleRectForEC(Rect in, const FrameHeader& frame_header, size_t ec) {
     return DivCeil(x * frame_header.upsampling,
                    frame_header.extra_channel_upsampling[ec]);
   };
+  // For x0 and y0 the DivCeil is actually an exact division.
   return Rect(s(in.x0()), s(in.y0()), s(in.xsize()), s(in.ysize()));
 }
 
@@ -574,6 +605,8 @@ Status FinalizeImageRect(
   Rect rect_for_if_storage = frame_rect;
   Rect rect_for_upsampling = frame_rect;
   Rect rect_for_if_input = input_rect;
+  // The same as rect_for_if_input but in the frame coordinates.
+  Rect frame_rect_for_ycbcr_upsampling = frame_rect;
   size_t extra_rows_t = 0;
   size_t extra_rows_b = 0;
   if (frame_header.upsampling != 1) {
@@ -619,6 +652,9 @@ Status FinalizeImageRect(
     rect_for_if_input =
         Rect(input_rect.x0() - ifbx0, input_rect.y0() - ifby0,
              rect_for_if_storage.xsize(), rect_for_if_storage.ysize());
+    frame_rect_for_ycbcr_upsampling =
+        Rect(frame_rect.x0() - ifbx0, frame_rect.y0() - ifby0,
+             rect_for_if_input.xsize(), rect_for_if_input.ysize());
     storage_for_if = &dec_state->upsampling_input_storage[thread];
   }
 
@@ -631,8 +667,13 @@ Status FinalizeImageRect(
     for (size_t c = 0; c < 3; c++) {
       size_t vs = frame_header.chroma_subsampling.VShift(c);
       size_t hs = frame_header.chroma_subsampling.HShift(c);
+      // The per-thread output is used for the first time here. Poison the temp
+      // image on this thread to prevent leaking initialized data from a
+      // previous run in this thread in msan builds.
+      msan::PoisonImage(dec_state->ycbcr_out_images[thread].Plane(c));
       HWY_DYNAMIC_DISPATCH(DoYCbCrUpsampling)
-      (hs, vs, &input_image->Plane(c), rect_for_if_input, frame_rect, frame_dim,
+      (hs, vs, &input_image->Plane(c), rect_for_if_input,
+       frame_rect_for_ycbcr_upsampling, frame_dim,
        &dec_state->ycbcr_out_images[thread].Plane(c), lf,
        &dec_state->ycbcr_temp_images[thread]);
     }
@@ -655,13 +696,8 @@ Status FinalizeImageRect(
   FilterPipeline* fp = nullptr;
   ssize_t ensure_padding_filter_y0 = 0;
   ssize_t ensure_padding_filter_y1 = 0;
-  Rect image_padded_rect;
   if (lf.epf_iters != 0 || lf.gab) {
     fp = &dec_state->filter_pipelines[thread];
-    size_t xextra =
-        rect_for_if_input.x0() % GroupBorderAssigner::kPaddingXRound;
-    image_padded_rect = Rect(rect_for_if.x0() - xextra, rect_for_if.y0(),
-                             rect_for_if.xsize() + xextra, rect_for_if.ysize());
   }
 
   // +----------------------------- STEP 2 ------------------------------+
@@ -804,27 +840,14 @@ Status FinalizeImageRect(
   // | Set up the filter pipeline.                                       |
   // +-------------------------------------------------------------------+
   if (fp) {
-    // If `rect_for_if_input` does not start at a multiple of
-    // GroupBorderAssigner::kPaddingXRound, we extend the rect we run EPF on by
-    // one full padding length to ensure sigma is handled correctly. We also
-    // extend the output and image rects accordingly. To do this, we need 2x the
-    // border.
-    size_t xextra =
-        rect_for_if_input.x0() % GroupBorderAssigner::kPaddingXRound;
-    Rect filter_input_padded_rect(
-        rect_for_if_input.x0() - xextra, rect_for_if_input.y0(),
-        rect_for_if_input.xsize() + xextra, rect_for_if_input.ysize());
     ensure_padding_filter.Init(
         input, rect_for_if_input, rect_for_if, frame_dim.xsize_padded,
         frame_dim.ysize_padded, lf.Padding(), lf.Padding(),
         &ensure_padding_filter_y0, &ensure_padding_filter_y1);
-    Rect filter_output_padded_rect(
-        rect_for_if_storage.x0() - xextra, rect_for_if_storage.y0(),
-        rect_for_if_storage.xsize() + xextra, rect_for_if_storage.ysize());
-    fp = PrepareFilterPipeline(dec_state, image_padded_rect, *input,
-                               filter_input_padded_rect, frame_dim.ysize_padded,
-                               thread, storage_for_if,
-                               filter_output_padded_rect);
+
+    fp = PrepareFilterPipeline(dec_state, rect_for_if, *input,
+                               rect_for_if_input, frame_dim.ysize_padded,
+                               thread, storage_for_if, rect_for_if_storage);
   }
 
   // +----------------------------- STEP 5 ------------------------------+
@@ -853,7 +876,7 @@ Status FinalizeImageRect(
       if (y >= first_ensure_padding_y && y < ensure_padding_filter_y1) {
         ensure_padding_filter.Process3(y);
       }
-      fp->ApplyFiltersRow(lf, dec_state->filter_weights, image_padded_rect, y);
+      fp->ApplyFiltersRow(lf, dec_state->filter_weights, y);
     } else if (output_pixel_data_storage != input) {
       for (size_t c = 0; c < 3; c++) {
         memcpy(rect_for_if_storage.PlaneRow(storage_for_if, c, y),
@@ -874,9 +897,9 @@ Status FinalizeImageRect(
     JXL_RETURN_IF_ERROR(image_features.patches.AddTo(
         storage_for_if, rect_for_if_storage.Line(available_y),
         ec_ptrs_for_patches.data(), rect_for_if.Line(available_y)));
-    JXL_RETURN_IF_ERROR(image_features.splines.AddTo(
-        storage_for_if, rect_for_if_storage.Line(available_y),
-        rect_for_if.Line(available_y), dec_state->shared->cmap));
+    image_features.splines.AddTo(storage_for_if,
+                                 rect_for_if_storage.Line(available_y),
+                                 rect_for_if.Line(available_y));
     size_t num_ys = 1;
     if (frame_header.upsampling != 1) {
       // Upsampling `y` values are relative to `rect_for_upsampling`, not to
@@ -985,7 +1008,8 @@ Status FinalizeImageRect(
           *output_pixel_data_storage,
           upsampled_frame_rect_for_storage.Lines(available_y, num_ys),
           upsampled_frame_rect.Lines(available_y, num_ys)
-              .Crop(Rect(0, 0, frame_dim.xsize, frame_dim.ysize)),
+              .Crop(Rect(0, 0, frame_dim.xsize_upsampled,
+                         frame_dim.ysize_upsampled)),
           alpha, alpha_rect.Lines(available_y, num_ys),
           dec_state->rgb_output_is_rgba, dec_state->rgb_output, frame_dim.xsize,
           dec_state->rgb_stride);
@@ -1019,9 +1043,9 @@ Status FinalizeImageRect(
         Rect alpha_line_rect = alpha_rect.Lines(available_y, num_ys);
         Rect color_input_line_rect =
             upsampled_frame_rect_for_storage.Lines(available_y, num_ys);
-        Rect image_line_rect =
-            upsampled_frame_rect.Lines(available_y, num_ys)
-                .Crop(Rect(0, 0, frame_dim.xsize, frame_dim.ysize));
+        Rect image_line_rect = upsampled_frame_rect.Lines(available_y, num_ys)
+                                   .Crop(Rect(0, 0, frame_dim.xsize_upsampled,
+                                              frame_dim.ysize_upsampled));
         const float* line_buffers[4];
         for (size_t iy = 0; iy < image_line_rect.ysize(); iy++) {
           for (size_t c = 0; c < 3; c++) {
@@ -1057,7 +1081,7 @@ Status FinalizeImageRect(
 
 Status FinalizeFrameDecoding(ImageBundle* decoded,
                              PassesDecoderState* dec_state, ThreadPool* pool,
-                             bool force_fir, bool skip_blending) {
+                             bool force_fir, bool skip_blending, bool move_ec) {
   const FrameHeader& frame_header = dec_state->shared->frame_header;
   const FrameDimensions& frame_dim = dec_state->shared->frame_dim;
 
@@ -1082,7 +1106,11 @@ Status FinalizeFrameDecoding(ImageBundle* decoded,
       const ImageMetadata& metadata = frame_header.nonserialized_metadata->m;
       for (size_t i = 0; i < metadata.num_extra_channels; i++) {
         if (frame_header.extra_channel_upsampling[i] == 1) {
-          ecs.push_back(std::move(dec_state->extra_channels[i]));
+          if (move_ec) {
+            ecs.push_back(std::move(dec_state->extra_channels[i]));
+          } else {
+            ecs.push_back(CopyImage(dec_state->extra_channels[i]));
+          }
         } else {
           ecs.emplace_back(frame_dim.xsize_upsampled_padded,
                            frame_dim.ysize_upsampled_padded);
@@ -1105,6 +1133,9 @@ Status FinalizeFrameDecoding(ImageBundle* decoded,
                 rects_to_process[rect_id].ysize() >>
                     frame_header.chroma_subsampling.VShift(c));
         Rect group_data_rect(xstart, ystart, rh.xsize(), rh.ysize());
+        // Poison the image in this thread to prevent leaking initialized data
+        // from a previous run in this thread in msan builds.
+        msan::PoisonImage(dec_state->group_data[thread].Plane(c));
         CopyImageToWithPadding(
             rh, dec_state->decoded.Plane(c), dec_state->FinalizeRectPadding(),
             group_data_rect, &dec_state->group_data[thread].Plane(c));
@@ -1121,10 +1152,15 @@ Status FinalizeFrameDecoding(ImageBundle* decoded,
               &dec_state
                    ->ec_temp_images[thread * decoded->extra_channels().size() +
                                     i];
+          // Poison the temp image on this thread to prevent leaking initialized
+          // data from a previous run in this thread in msan builds.
+          msan::PoisonImage(*eti);
+          JXL_CHECK_IMAGE_INITIALIZED(dec_state->extra_channels[i], r);
           CopyImageToWithPadding(r, dec_state->extra_channels[i],
                                  /*padding=*/2, ec_input_rect, eti);
           ec_rects.emplace_back(eti, ec_input_rect);
         } else {
+          JXL_CHECK_IMAGE_INITIALIZED(decoded->extra_channels()[i], r);
           ec_rects.emplace_back(&decoded->extra_channels()[i], r);
         }
       }
