@@ -29,6 +29,8 @@ namespace {
 // TODO(eustas): this is a poor-mans replacement for memory-manager approach;
 //               remove, once memory-manager actually works.
 size_t memory_limit_base_ = 0;
+size_t cpu_limit_base_ = 0;
+size_t used_cpu_base_ = 0;
 
 bool CheckSizeLimit(size_t xsize, size_t ysize) {
   if (!memory_limit_base_) return true;
@@ -362,6 +364,13 @@ std::vector<size_t> GetFrameDependencies(size_t index,
   return result;
 }
 
+// Parameters for user-requested extra channel output.
+struct ExtraChannelOutput {
+  JxlPixelFormat format;
+  void* buffer;
+  size_t buffer_size;
+};
+
 }  // namespace
 
 // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
@@ -436,10 +445,13 @@ struct JxlDecoderStruct {
   size_t preview_out_size;
   size_t image_out_size;
 
-  // TODO(lode): merge these?
   JxlPixelFormat preview_out_format;
   JxlPixelFormat image_out_format;
 
+  // For extra channels. Empty if no extra channels are requested, and they are
+  // reset each frame
+  std::vector<ExtraChannelOutput> extra_channel_output;
+
   jxl::CodecMetadata metadata;
   std::unique_ptr<jxl::ImageBundle> ib;
   // ColorEncoding to use for xyb encoded image with ICC profile.
@@ -525,8 +537,8 @@ JxlDecoderStatus JxlDecoderDefaultPixelFormat(const JxlDecoder* dec,
   return JXL_DEC_SUCCESS;
 }
 
-void JxlDecoderReset(JxlDecoder* dec) {
-  dec->thread_pool.reset();
+// Resets the state that must be reset for both Rewind and Reset
+void JxlDecoderRewindDecodingState(JxlDecoder* dec) {
   dec->stage = DecoderStage::kInited;
   dec->got_signature = false;
   dec->first_codestream_seen = false;
@@ -544,9 +556,7 @@ void JxlDecoderReset(JxlDecoder* dec) {
   dec->codestream_pos = 0;
   dec->codestream_begin = 0;
   dec->codestream_end = 0;
-  dec->keep_orientation = false;
   dec->events_wanted = 0;
-  dec->orig_events_wanted = 0;
   dec->basic_info_size_hint = InitialBasicInfoSizeHint();
   dec->have_container = 0;
   dec->preview_out_buffer_set = false;
@@ -557,6 +567,7 @@ void JxlDecoderReset(JxlDecoder* dec) {
   dec->image_out_opaque = nullptr;
   dec->preview_out_size = 0;
   dec->image_out_size = 0;
+  dec->extra_channel_output.clear();
   dec->dec_pixels = 0;
   dec->next_in = 0;
   dec->avail_in = 0;
@@ -580,6 +591,14 @@ void JxlDecoderReset(JxlDecoder* dec) {
   dec->skipping_frame = false;
   dec->internal_frames = 0;
   dec->external_frames = 0;
+}
+
+void JxlDecoderReset(JxlDecoder* dec) {
+  JxlDecoderRewindDecodingState(dec);
+
+  dec->thread_pool.reset();
+  dec->keep_orientation = false;
+  dec->orig_events_wanted = 0;
   dec->frame_references.clear();
   dec->frame_saved_as.clear();
   dec->frame_external_to_internal.clear();
@@ -611,27 +630,7 @@ void JxlDecoderDestroy(JxlDecoder* dec) {
   }
 }
 
-void JxlDecoderRewind(JxlDecoder* dec) {
-  int keep_orientation = dec->keep_orientation;
-  int events_wanted = dec->orig_events_wanted;
-  std::vector<int> frame_references;
-  std::vector<int> frame_saved_as;
-  std::vector<size_t> frame_external_to_internal;
-  std::vector<char> frame_required;
-  frame_references.swap(dec->frame_references);
-  frame_saved_as.swap(dec->frame_saved_as);
-  frame_external_to_internal.swap(dec->frame_external_to_internal);
-  frame_required.swap(dec->frame_required);
-
-  JxlDecoderReset(dec);
-  dec->keep_orientation = keep_orientation;
-  dec->events_wanted = events_wanted;
-  dec->orig_events_wanted = events_wanted;
-  frame_references.swap(dec->frame_references);
-  frame_saved_as.swap(dec->frame_saved_as);
-  frame_external_to_internal.swap(dec->frame_external_to_internal);
-  frame_required.swap(dec->frame_required);
-}
+void JxlDecoderRewind(JxlDecoder* dec) { JxlDecoderRewindDecodingState(dec); }
 
 void JxlDecoderSkipFrames(JxlDecoder* dec, size_t amount) {
   // Increment amount, rather than set it: making the amount smaller is
@@ -665,7 +664,9 @@ void JxlDecoderSkipFrames(JxlDecoder* dec, size_t amount) {
 JXL_EXPORT JxlDecoderStatus
 JxlDecoderSetParallelRunner(JxlDecoder* dec, JxlParallelRunner parallel_runner,
                             void* parallel_runner_opaque) {
-  if (dec->thread_pool) return JXL_API_ERROR("parallel runner already set");
+  if (dec->stage != DecoderStage::kInited) {
+    return JXL_API_ERROR("parallel_runner must be set before starting");
+  }
   dec->thread_pool.reset(
       new jxl::ThreadPool(parallel_runner, parallel_runner_opaque));
   return JXL_DEC_SUCCESS;
@@ -809,6 +810,8 @@ JxlDecoderStatus JxlDecoderReadAllHeaders(JxlDecoder* dec, const uint8_t* in,
     ImageMetadata dummy_metadata;
     JXL_API_RETURN_IF_ERROR(ReadBundle(span, reader.get(), &dummy_metadata));
 
+    dec->metadata.transform_data.nonserialized_xyb_encoded =
+        dec->metadata.m.xyb_encoded;
     JXL_API_RETURN_IF_ERROR(
         ReadBundle(span, reader.get(), &dec->metadata.transform_data));
   }
@@ -880,16 +883,19 @@ static size_t GetStride(const JxlDecoder* dec, const JxlPixelFormat& format,
   return stride;
 }
 
-static JxlDecoderStatus ConvertImageInternal(const JxlDecoder* dec,
-                                             const jxl::ImageBundle& frame,
-                                             const JxlPixelFormat& format,
-                                             void* out_image, size_t out_size,
-                                             JxlImageOutCallback out_callback,
-                                             void* out_opaque) {
+// Internal wrapper around jxl::ConvertToExternal which converts the stride,
+// format and orientation and allows to choose whether to get all RGB(A)
+// channels or alternatively get a single extra channel.
+// If want_extra_channel, a valid index to a single extra channel must be
+// given, the output must be single-channel, and format.num_channels is ignored
+// and treated as if it is 1.
+static JxlDecoderStatus ConvertImageInternal(
+    const JxlDecoder* dec, const jxl::ImageBundle& frame,
+    const JxlPixelFormat& format, bool want_extra_channel,
+    size_t extra_channel_index, void* out_image, size_t out_size,
+    JxlImageOutCallback out_callback, void* out_opaque) {
   // TODO(lode): handle mismatch of RGB/grayscale color profiles and pixel data
   // color/grayscale format
-  const auto& metadata = dec->metadata.m;
-
   const size_t stride = GetStride(dec, format, &frame);
 
   bool float_format = format.data_type == JXL_TYPE_FLOAT ||
@@ -897,13 +903,24 @@ static JxlDecoderStatus ConvertImageInternal(const JxlDecoder* dec,
 
   jxl::Orientation undo_orientation = dec->keep_orientation
                                           ? jxl::Orientation::kIdentity
-                                          : metadata.GetOrientation();
-  JXL_DASSERT(!dec->frame_dec || !dec->frame_dec->HasRGBBuffer());
-  jxl::Status status = jxl::ConvertToExternal(
-      frame, BitsPerChannel(format.data_type), float_format,
-      format.num_channels, format.endianness, stride, dec->thread_pool.get(),
-      out_image, out_size, /*out_callback=*/out_callback,
-      /*out_opaque=*/out_opaque, undo_orientation);
+                                          : dec->metadata.m.GetOrientation();
+
+  jxl::Status status(true);
+  if (want_extra_channel) {
+    status = jxl::ConvertToExternal(
+        frame.extra_channels()[extra_channel_index],
+        BitsPerChannel(format.data_type), float_format, format.endianness,
+        stride, dec->thread_pool.get(), out_image, out_size,
+        /*out_callback=*/out_callback,
+        /*out_opaque=*/out_opaque, undo_orientation);
+  } else {
+    status = jxl::ConvertToExternal(
+        frame, BitsPerChannel(format.data_type), float_format,
+        format.num_channels, format.endianness, stride, dec->thread_pool.get(),
+        out_image, out_size,
+        /*out_callback=*/out_callback,
+        /*out_opaque=*/out_opaque, undo_orientation);
+  }
 
   return status ? JXL_DEC_SUCCESS : JXL_DEC_ERROR;
 }
@@ -1071,7 +1088,8 @@ JxlDecoderStatus JxlDecoderProcessInternal(JxlDecoder* dec, const uint8_t* in,
       if (want_preview) {
         if (dec->preview_out_buffer) {
           JxlDecoderStatus status = ConvertImageInternal(
-              dec, ib, dec->preview_out_format, dec->preview_out_buffer,
+              dec, ib, dec->preview_out_format, /*want_extra_channel=*/false,
+              /*extra_channel_index=*/0, dec->preview_out_buffer,
               dec->preview_out_size, /*out_callback=*/nullptr,
               /*out_opaque=*/nullptr);
           if (status != JXL_DEC_SUCCESS) return status;
@@ -1197,7 +1215,7 @@ JxlDecoderStatus JxlDecoderProcessInternal(JxlDecoder* dec, const uint8_t* in,
 
       jxl::Status status = dec->frame_dec->InitFrame(
           reader.get(), dec->ib.get(), /*is_preview=*/false,
-          /*allow_partial_frames=*/false, /*allow_partial_dc_global=*/false);
+          /*allow_partial_frames=*/true, /*allow_partial_dc_global=*/false);
       if (!status) JXL_API_RETURN_IF_ERROR(status);
 
       size_t sections_begin =
@@ -1236,7 +1254,8 @@ JxlDecoderStatus JxlDecoderProcessInternal(JxlDecoder* dec, const uint8_t* in,
 
       if (dec->image_out_buffer_set && !!dec->image_out_buffer &&
           dec->image_out_format.data_type == JXL_TYPE_UINT8 &&
-          dec->image_out_format.num_channels >= 3) {
+          dec->image_out_format.num_channels >= 3 &&
+          dec->extra_channel_output.empty()) {
         bool is_rgba = dec->image_out_format.num_channels == 4;
         dec->frame_dec->MaybeSetRGB8OutputBuffer(
             reinterpret_cast<uint8_t*>(dec->image_out_buffer),
@@ -1270,6 +1289,20 @@ JxlDecoderStatus JxlDecoderProcessInternal(JxlDecoder* dec, const uint8_t* in,
         return JXL_DEC_NEED_MORE_INPUT;
       }
       dec->sections->SetInput(in + pos, size - pos);
+
+      if (cpu_limit_base_ != 0) {
+        FrameDimensions frame_dim = dec->frame_header->ToFrameDimensions();
+        // No overflow, checked in ParseHeader.
+        size_t num_pixels = frame_dim.xsize * frame_dim.ysize;
+        if (used_cpu_base_ + num_pixels < used_cpu_base_) {
+          return JXL_API_ERROR("used too much CPU");
+        }
+        used_cpu_base_ += num_pixels;
+        if (used_cpu_base_ > cpu_limit_base_) {
+          return JXL_API_ERROR("used too much CPU");
+        }
+      }
+
       jxl::Status status =
           dec->frame_dec->ProcessSections(dec->sections->section_info.data(),
                                           dec->sections->section_info.size(),
@@ -1323,12 +1356,28 @@ JxlDecoderStatus JxlDecoderProcessInternal(JxlDecoder* dec, const uint8_t* in,
           if (!dec->frame_dec->HasRGBBuffer()) {
             // Copy pixels if desired.
             JxlDecoderStatus status = ConvertImageInternal(
-                dec, *dec->ib, dec->image_out_format, dec->image_out_buffer,
+                dec, *dec->ib, dec->image_out_format,
+                /*want_extra_channel=*/false,
+                /*extra_channel_index=*/0, dec->image_out_buffer,
                 dec->image_out_size, dec->image_out_callback,
                 dec->image_out_opaque);
             if (status != JXL_DEC_SUCCESS) return status;
           }
           dec->image_out_buffer_set = false;
+
+          for (size_t i = 0; i < dec->extra_channel_output.size(); ++i) {
+            void* buffer = dec->extra_channel_output[i].buffer;
+            // buffer nullptr indicates this extra channel is not requested
+            if (!buffer) continue;
+            const JxlPixelFormat* format = &dec->extra_channel_output[i].format;
+            JxlDecoderStatus status = ConvertImageInternal(
+                dec, *dec->ib, *format,
+                /*want_extra_channel=*/true, i, buffer,
+                dec->extra_channel_output[i].buffer_size, nullptr, nullptr);
+            if (status != JXL_DEC_SUCCESS) return status;
+          }
+
+          dec->extra_channel_output.clear();
         }
       }
     }
@@ -1712,6 +1761,10 @@ JxlDecoderStatus JxlDecoderProcessInput(JxlDecoder* dec) {
   return result;
 }
 
+// To ensure ABI forward-compatibility, this struct has a constant size.
+static_assert(sizeof(JxlBasicInfo) == 204,
+              "JxlBasicInfo struct size should remain constant");
+
 JxlDecoderStatus JxlDecoderGetBasicInfo(const JxlDecoder* dec,
                                         JxlBasicInfo* info) {
   if (!dec->got_basic_info) return JXL_DEC_NEED_MORE_INPUT;
@@ -1796,7 +1849,7 @@ JxlDecoderStatus JxlDecoderGetExtraChannelInfo(const JxlDecoder* dec,
           : 0;
   info->dim_shift = channel.dim_shift;
   info->name_length = channel.name.size();
-  info->alpha_associated = channel.alpha_associated;
+  info->alpha_premultiplied = channel.alpha_associated;
   info->spot_color[0] = channel.spot_color[0];
   info->spot_color[1] = channel.spot_color[1];
   info->spot_color[2] = channel.spot_color[2];
@@ -1927,9 +1980,6 @@ JxlDecoderStatus PrepareSizeCheck(const JxlDecoder* dec,
   if (format->num_channels > 4) {
     return JXL_API_ERROR("More than 4 channels not supported");
   }
-  if (format->num_channels < 3 && !dec->metadata.m.color_encoding.IsGray()) {
-    return JXL_API_ERROR("Grayscale output not possible for color image");
-  }
   if (format->data_type == JXL_TYPE_BOOLEAN) {
     return JXL_API_ERROR("Boolean data type not yet supported");
   }
@@ -1956,19 +2006,10 @@ JxlDecoderStatus JxlDecoderFlushImage(JxlDecoder* dec) {
     return JXL_DEC_ERROR;
   }
   if (!dec->frame_dec->HasDecodedDC()) {
-    // FrameDecoder::Fush currently requires DC to have been decoded already
+    // FrameDecoder::Flush currently requires DC to have been decoded already
     // to work correctly.
     return JXL_DEC_ERROR;
   }
-  if (dec->frame_header->encoding != jxl::FrameEncoding::kVarDCT) {
-    // Flushing does not yet work correctly if the frame uses modular encoding.
-    return JXL_DEC_ERROR;
-  }
-  if (dec->metadata.m.num_extra_channels > 0) {
-    // Flushing does not yet work correctly if there are extra channels, which
-    // use modular
-    return JXL_DEC_ERROR;
-  }
 
   if (!dec->frame_dec->Flush()) {
     return JXL_DEC_ERROR;
@@ -1984,8 +2025,9 @@ JxlDecoderStatus JxlDecoderFlushImage(JxlDecoder* dec) {
   size_t ysize = dec->ib->ysize();
   dec->ib->ShrinkTo(dec->metadata.size.xsize(), dec->metadata.size.ysize());
   JxlDecoderStatus status = jxl::ConvertImageInternal(
-      dec, *dec->ib, dec->image_out_format, dec->image_out_buffer,
-      dec->image_out_size,
+      dec, *dec->ib, dec->image_out_format,
+      /*want_extra_channel=*/false,
+      /*extra_channel_index=*/0, dec->image_out_buffer, dec->image_out_size,
       /*out_callback=*/nullptr, /*out_opaque=*/nullptr);
   dec->ib->ShrinkTo(xsize, ysize);
   if (status != JXL_DEC_SUCCESS) return status;
@@ -1997,6 +2039,9 @@ JXL_EXPORT JxlDecoderStatus JxlDecoderPreviewOutBufferSize(
   size_t bits;
   JxlDecoderStatus status = PrepareSizeCheck(dec, format, &bits);
   if (status != JXL_DEC_SUCCESS) return status;
+  if (format->num_channels < 3 && !dec->metadata.m.color_encoding.IsGray()) {
+    return JXL_API_ERROR("Grayscale output not possible for color image");
+  }
 
   size_t xsize = dec->metadata.oriented_preview_xsize(dec->keep_orientation);
   size_t ysize = dec->metadata.oriented_preview_ysize(dec->keep_orientation);
@@ -2016,6 +2061,9 @@ JXL_EXPORT JxlDecoderStatus JxlDecoderSetPreviewOutBuffer(
       !(dec->orig_events_wanted & JXL_DEC_PREVIEW_IMAGE)) {
     return JXL_API_ERROR("No preview out buffer needed at this time");
   }
+  if (format->num_channels < 3 && !dec->metadata.m.color_encoding.IsGray()) {
+    return JXL_API_ERROR("Grayscale output not possible for color image");
+  }
 
   size_t min_size;
   // This also checks whether the format is valid and supported and basic info
@@ -2065,6 +2113,9 @@ JXL_EXPORT JxlDecoderStatus JxlDecoderImageOutBufferSize(
   size_t bits;
   JxlDecoderStatus status = PrepareSizeCheck(dec, format, &bits);
   if (status != JXL_DEC_SUCCESS) return status;
+  if (format->num_channels < 3 && !dec->metadata.m.color_encoding.IsGray()) {
+    return JXL_API_ERROR("Grayscale output not possible for color image");
+  }
 
   size_t row_size =
       jxl::DivCeil(dec->metadata.oriented_xsize(dec->keep_orientation) *
@@ -2088,6 +2139,9 @@ JxlDecoderStatus JxlDecoderSetImageOutBuffer(JxlDecoder* dec,
     return JXL_API_ERROR(
         "Cannot change from image out callback to image out buffer");
   }
+  if (format->num_channels < 3 && !dec->metadata.m.color_encoding.IsGray()) {
+    return JXL_API_ERROR("Grayscale output not possible for color image");
+  }
   size_t min_size;
   // This also checks whether the format is valid and supported and basic info
   // is available.
@@ -2105,6 +2159,63 @@ JxlDecoderStatus JxlDecoderSetImageOutBuffer(JxlDecoder* dec,
   return JXL_DEC_SUCCESS;
 }
 
+JxlDecoderStatus JxlDecoderExtraChannelBufferSize(const JxlDecoder* dec,
+                                                  const JxlPixelFormat* format,
+                                                  size_t* size,
+                                                  uint32_t index) {
+  if (!dec->got_basic_info || !(dec->orig_events_wanted & JXL_DEC_FULL_IMAGE)) {
+    return JXL_API_ERROR("No extra channel buffer needed at this time");
+  }
+
+  if (index >= dec->metadata.m.num_extra_channels) {
+    return JXL_API_ERROR("Invalid extra channel index");
+  }
+
+  size_t num_channels = 1;  // Do not use format's num_channels
+
+  size_t bits;
+  JxlDecoderStatus status = PrepareSizeCheck(dec, format, &bits);
+  if (status != JXL_DEC_SUCCESS) return status;
+
+  size_t row_size = jxl::DivCeil(
+      dec->metadata.oriented_xsize(dec->keep_orientation) * num_channels * bits,
+      jxl::kBitsPerByte);
+  if (format->align > 1) {
+    row_size = jxl::DivCeil(row_size, format->align) * format->align;
+  }
+  *size = row_size * dec->metadata.oriented_ysize(dec->keep_orientation);
+
+  return JXL_DEC_SUCCESS;
+}
+
+JxlDecoderStatus JxlDecoderSetExtraChannelBuffer(JxlDecoder* dec,
+                                                 const JxlPixelFormat* format,
+                                                 void* buffer, size_t size,
+                                                 uint32_t index) {
+  size_t min_size;
+  // This also checks whether the format and index are valid and supported and
+  // basic info is available.
+  JxlDecoderStatus status =
+      JxlDecoderExtraChannelBufferSize(dec, format, &min_size, index);
+  if (status != JXL_DEC_SUCCESS) return status;
+
+  if (size < min_size) return JXL_DEC_ERROR;
+
+  if (dec->extra_channel_output.size() <= index) {
+    dec->extra_channel_output.resize(dec->metadata.m.num_extra_channels,
+                                     {{}, nullptr, 0});
+  }
+  // Guaranteed correct thanks to check in JxlDecoderExtraChannelBufferSize.
+  JXL_ASSERT(index < dec->extra_channel_output.size());
+
+  dec->extra_channel_output[index].format = *format;
+  dec->extra_channel_output[index].format.num_channels = 1;
+  dec->extra_channel_output[index].buffer = buffer;
+  dec->extra_channel_output[index].buffer_size = size;
+
+  return JXL_DEC_SUCCESS;
+}
+
 JxlDecoderStatus JxlDecoderSetImageOutCallback(JxlDecoder* dec,
                                                const JxlPixelFormat* format,
                                                JxlImageOutCallback callback,
@@ -2183,6 +2294,14 @@ JxlDecoderStatus JxlDecoderSetPreferredColorProfile(
   return JXL_DEC_SUCCESS;
 }
 
+// This function is "package-private". It is only used by fuzzer to avoid
+// running cases that are too memory / CPU hungry. Limitations are applied
+// at mid-level API. In the future high-level API would also include the
+// means of limiting / throttling memory / CPU usage.
 void SetDecoderMemoryLimitBase_(size_t memory_limit_base) {
   memory_limit_base_ = memory_limit_base;
+  // Allow 5 x max_image_size processing units; every frame is accounted
+  // as W x H CPU processing units, so there could be numerous small frames
+  // or few larger ones.
+  cpu_limit_base_ = 5 * memory_limit_base;
 }
