@@ -28,6 +28,7 @@
 #include "lib/jxl/frame_header.h"
 #include "lib/jxl/gaborish.h"
 #include "lib/jxl/modular/encoding/context_predict.h"
+#include "lib/jxl/modular/encoding/enc_debug_tree.h"
 #include "lib/jxl/modular/encoding/enc_encoding.h"
 #include "lib/jxl/modular/encoding/encoding.h"
 #include "lib/jxl/modular/encoding/ma_common.h"
@@ -391,8 +392,8 @@ ModularFrameEncoder::ModularFrameEncoder(const FrameHeader& frame_header,
       // TODO(veluca): allow all predictors that don't break residual
       // multipliers in lossy mode.
       cparams.options.predictor = Predictor::Variable;
-    } else if (cparams.responsive) {
-      // zero predictor for Squeeze residues
+    } else if (cparams.responsive || cparams.lossy_palette) {
+      // zero predictor for Squeeze residues and lossy palette
       cparams.options.predictor = Predictor::Zero;
     } else if (quality < 100) {
       // If not responsive and lossy. TODO(veluca): use near_lossless instead?
@@ -407,6 +408,9 @@ ModularFrameEncoder::ModularFrameEncoder(const FrameHeader& frame_header,
       // just gradient predictor in thunder mode
       cparams.options.predictor = Predictor::Gradient;
     }
+  } else {
+    delta_pred = cparams.options.predictor;
+    if (cparams.lossy_palette) cparams.options.predictor = Predictor::Zero;
   }
   tree_splits.push_back(0);
   if (cparams.modular_mode == false) {
@@ -468,7 +472,8 @@ Status ModularFrameEncoder::ComputeEncodingData(
 
   nb_chans += extra_channels.size();
 
-  bool fp = metadata.bit_depth.floating_point_sample;
+  bool fp = metadata.bit_depth.floating_point_sample &&
+            cparams.color_transform != ColorTransform::kXYB;
 
   // bits_per_sample is just metadata for XYB images.
   if (metadata.bit_depth.bits_per_sample >= 32 && do_color &&
@@ -638,7 +643,7 @@ Status ModularFrameEncoder::ComputeEncodingData(
       maybe_palette.lossy_palette =
           (cparams.lossy_palette && maybe_palette.num_c == 3);
       if (maybe_palette.lossy_palette) {
-        maybe_palette.predictor = Predictor::Average4;
+        maybe_palette.predictor = delta_pred;
       }
       // TODO(veluca): use a custom weighted header if using the weighted
       // predictor.
@@ -655,7 +660,7 @@ Status ModularFrameEncoder::ComputeEncodingData(
       maybe_palette_3.ordered_palette = cparams.palette_colors >= 0;
       maybe_palette_3.lossy_palette = cparams.lossy_palette;
       if (maybe_palette_3.lossy_palette) {
-        maybe_palette_3.predictor = Predictor::Average4;
+        maybe_palette_3.predictor = delta_pred;
       }
       do_transform(gi, maybe_palette_3, weighted::Header(), pool);
     }
@@ -679,8 +684,9 @@ Status ModularFrameEncoder::ComputeEncodingData(
   }
 
   if (cparams.responsive && !gi.channel.empty()) {
-    do_transform(gi, Transform(TransformId::kSqueeze), weighted::Header(),
-                 pool);  // use default squeezing
+    Transform t(TransformId::kSqueeze);
+    t.squeezes = cparams.squeezes;
+    do_transform(gi, t, weighted::Header(), pool);
   }
 
   std::vector<uint32_t> quants;
@@ -731,10 +737,9 @@ Status ModularFrameEncoder::ComputeEncodingData(
       // assuming default Squeeze here
       int component = ((i - gi.nb_meta_channels) % nb_chans);
       // last 4 channels are final chroma residuals
-      if (nb_chans > 2 && i >= gi.channel.size() - 4) {
+      if (nb_chans > 2 && i >= gi.channel.size() - 4 && cparams.responsive) {
         component = 1;
       }
-
       if (cparams.color_transform == ColorTransform::kXYB && component < 3) {
         q = (component == 0 ? quality : cquality) * squeeze_quality_factor_xyb *
             squeeze_xyb_qtable[component][shift];
@@ -838,8 +843,8 @@ Status ModularFrameEncoder::ComputeEncodingData(
             multiplier_info.back().range[1][0] != stream_id ||
             multiplier_info.back().multiplier != q) {
           StaticPropRange range;
-          range[0] = {i, i + 1};
-          range[1] = {stream_id, stream_id + 1};
+          range[0] = {{i, i + 1}};
+          range[1] = {{stream_id, stream_id + 1}};
           multiplier_info.push_back({range, (uint32_t)q});
         } else {
           // Previous channel in the same group had the same quantization
@@ -956,8 +961,8 @@ Status ModularFrameEncoder::PrepareEncoding(ThreadPool* pool,
                                 pixel_samples, diff_samples);
           }
           StaticPropRange range;
-          range[0] = {0, max_c};
-          range[1] = {start, stop};
+          range[0] = {{0, max_c}};
+          range[1] = {{start, stop}};
           auto local_multiplier_info = multiplier_info;
 
           tree_samples.PreQuantizeProperties(
@@ -1328,20 +1333,18 @@ Status ModularFrameEncoder::PrepareStreamParams(const Rect& rect,
                   1 * 7 + 2, 2 * 7 + 1, 2 * 7 + 2, 2 * 7 + 3, 4 * 7 + 4,
                   4 * 7 + 5, 0 * 7 + 2, 0 * 7 + 1, 0 * 7 + 3}) {
       if (nb_rcts_to_try == 0) break;
-      int num_transforms_to_keep = gi.transform.size();
       sg.rct_type = i;
-      do_transform(gi, sg, weighted::Header());
-      float cost = EstimateCost(gi);
-      if (cost < best_cost) {
-        best_rct = i;
-        best_cost = cost;
-      }
       nb_rcts_to_try--;
-      // Ensure we do not clamp channels to their supposed range, as this
-      // otherwise breaks in the presence of patches.
-      gi.undo_transforms(weighted::Header(), num_transforms_to_keep == 0
-                                                 ? -1
-                                                 : num_transforms_to_keep);
+      if (do_transform(gi, sg, weighted::Header())) {
+        float cost = EstimateCost(gi);
+        if (cost < best_cost) {
+          best_rct = i;
+          best_cost = cost;
+        }
+        Transform t = gi.transform.back();
+        JXL_RETURN_IF_ERROR(t.Inverse(gi, weighted::Header(), nullptr));
+        gi.transform.pop_back();
+      }
     }
     // Apply the best RCT to the image for future encoding.
     sg.rct_type = best_rct;
