@@ -21,6 +21,7 @@
 #include "lib/jxl/base/padded_bytes.h"
 #include "lib/jxl/base/thread_pool_internal.h"
 #include "lib/jxl/codec_in_out.h"
+#include "lib/jxl/codec_y4m_testonly.h"
 #include "lib/jxl/color_encoding_internal.h"
 #include "lib/jxl/color_management.h"
 #include "lib/jxl/dec_file.h"
@@ -66,7 +67,7 @@ TEST(JxlTest, HeaderSize) {
     CodecInOut io2;
     AuxOut aux_out;
     Roundtrip(&io, cparams, dparams, pool, &io2, &aux_out);
-    EXPECT_LE(aux_out.layers[kLayerHeader].total_bits, 34);
+    EXPECT_LE(aux_out.layers[kLayerHeader].total_bits, 34u);
   }
 
   {
@@ -77,7 +78,7 @@ TEST(JxlTest, HeaderSize) {
     io.Main().SetAlpha(std::move(alpha), /*alpha_is_premultiplied=*/false);
     AuxOut aux_out;
     Roundtrip(&io, cparams, dparams, pool, &io2, &aux_out);
-    EXPECT_LE(aux_out.layers[kLayerHeader].total_bits, 57);
+    EXPECT_LE(aux_out.layers[kLayerHeader].total_bits, 46u);
   }
 }
 
@@ -152,7 +153,7 @@ TEST(JxlTest, RoundtripSmallD1) {
     io.ShrinkTo(io.xsize() / 8, io.ysize() / 8);
 
     compressed_size = Roundtrip(&io, cparams, dparams, pool, &io_out);
-    EXPECT_LE(compressed_size, 1000);
+    EXPECT_LE(compressed_size, 1000u);
     EXPECT_LE(ButteraugliDistance(io, io_out, cparams.ba_params,
                                   /*distmap=*/nullptr, pool),
               1.5);
@@ -192,7 +193,7 @@ TEST(JxlTest, RoundtripOtherTransforms) {
   std::unique_ptr<CodecInOut> io2 = jxl::make_unique<CodecInOut>();
   const size_t compressed_size =
       Roundtrip(io.get(), cparams, dparams, pool, io2.get());
-  EXPECT_LE(compressed_size, 23000);
+  EXPECT_LE(compressed_size, 23000u);
   EXPECT_LE(ButteraugliDistance(*io, *io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             6);
@@ -201,7 +202,7 @@ TEST(JxlTest, RoundtripOtherTransforms) {
   std::unique_ptr<CodecInOut> io3 = jxl::make_unique<CodecInOut>();
   const size_t compressed_size2 =
       Roundtrip(io.get(), cparams, dparams, pool, io3.get());
-  EXPECT_LE(compressed_size2, 23000);
+  EXPECT_LE(compressed_size2, 23000u);
   EXPECT_LE(ButteraugliDistance(*io, *io3, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             6);
@@ -218,10 +219,10 @@ TEST(JxlTest, RoundtripResample2) {
   cparams.resampling = 2;
   DecompressParams dparams;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 15777);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 17000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
-            12.5);
+            12);
 }
 TEST(JxlTest, RoundtripResample2MT) {
   ThreadPoolInternal pool(4);
@@ -234,13 +235,15 @@ TEST(JxlTest, RoundtripResample2MT) {
   cparams.resampling = 2;
   DecompressParams dparams;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 57000);
+  // TODO(veluca): Figure out why msan and release produce different
+  // file size.
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 64500u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, &pool),
 #if JXL_HIGH_PRECISION
-            4.5);
+            5.5);
 #else
-            12.5);
+            13.5);
 #endif
 }
 
@@ -255,7 +258,7 @@ TEST(JxlTest, RoundtripResample4) {
   cparams.resampling = 4;
   DecompressParams dparams;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 6000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 6000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             28);
@@ -272,7 +275,7 @@ TEST(JxlTest, RoundtripResample8) {
   cparams.resampling = 8;
   DecompressParams dparams;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2100);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2100u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             80);
@@ -291,7 +294,7 @@ TEST(JxlTest, RoundtripUnalignedD2) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 700);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 700u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             3.2);
@@ -320,7 +323,7 @@ TEST(JxlTest, RoundtripMultiGroupNL) {
 
   cparams.butteraugli_distance = 2.0f;
   CodecInOut io3;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 80000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 80000u);
   EXPECT_LE(ButteraugliDistance(io, io3, cparams.ba_params,
                                 /*distmap=*/nullptr, &pool),
             1.5f);
@@ -342,14 +345,14 @@ TEST(JxlTest, RoundtripMultiGroup) {
   cparams.butteraugli_distance = 1.0f;
   cparams.speed_tier = SpeedTier::kKitten;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 40000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 40000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, &pool),
             1.99f);
 
   cparams.butteraugli_distance = 2.0f;
   CodecInOut io3;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 22100);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 22100u);
   EXPECT_LE(ButteraugliDistance(io, io3, cparams.ba_params,
                                 /*distmap=*/nullptr, &pool),
             3.0f);
@@ -367,7 +370,7 @@ TEST(JxlTest, RoundtripLargeFast) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 265000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 265000u);
 }
 
 TEST(JxlTest, RoundtripDotsForceEpf) {
@@ -384,7 +387,7 @@ TEST(JxlTest, RoundtripDotsForceEpf) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 265000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 265000u);
 }
 
 // Checks for differing size/distance in two consecutive runs of distance 2,
@@ -470,7 +473,7 @@ TEST(JxlTest, RoundtripSmallNL) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 1500);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 1500u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             1.7);
@@ -492,7 +495,7 @@ TEST(JxlTest, RoundtripNoGaborishNoAR) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 40000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 40000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             2.5);
@@ -512,7 +515,7 @@ TEST(JxlTest, RoundtripSmallNoGaborish) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 900);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 900u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             1.7);
@@ -544,7 +547,7 @@ TEST(JxlTest, RoundtripSmallPatchesAlpha) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             0.5f);
@@ -572,7 +575,7 @@ TEST(JxlTest, RoundtripSmallPatches) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             0.5f);
@@ -607,8 +610,8 @@ TEST(JxlTest, RoundtripImageBundleOriginalBits) {
 
     EXPECT_EQ(bit_depth, io2.metadata.m.bit_depth.bits_per_sample);
     EXPECT_FALSE(io2.metadata.m.bit_depth.floating_point_sample);
-    EXPECT_EQ(0, io2.metadata.m.bit_depth.exponent_bits_per_sample);
-    EXPECT_EQ(0, io2.metadata.m.GetAlphaBits());
+    EXPECT_EQ(0u, io2.metadata.m.bit_depth.exponent_bits_per_sample);
+    EXPECT_EQ(0u, io2.metadata.m.GetAlphaBits());
   }
 
   // Test various existing and non-existing floating point formats
@@ -646,7 +649,7 @@ TEST(JxlTest, RoundtripImageBundleOriginalBits) {
     EXPECT_TRUE(io2.metadata.m.bit_depth.floating_point_sample);
     EXPECT_EQ(exponent_bit_depth,
               io2.metadata.m.bit_depth.exponent_bits_per_sample);
-    EXPECT_EQ(0, io2.metadata.m.GetAlphaBits());
+    EXPECT_EQ(0u, io2.metadata.m.GetAlphaBits());
   }
 }
 
@@ -656,12 +659,12 @@ TEST(JxlTest, RoundtripGrayscale) {
       ReadTestData("wesaturate/500px/cvo9xd_keong_macan_grayscale.png");
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
   io.ShrinkTo(128, 128);
   EXPECT_TRUE(io.Main().IsGray());
-  EXPECT_EQ(8, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(8u, io.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
   EXPECT_TRUE(io.metadata.m.color_encoding.tf.IsSRGB());
 
   PassesEncoderState enc_state;
@@ -679,7 +682,7 @@ TEST(JxlTest, RoundtripGrayscale) {
     EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
     EXPECT_TRUE(io2.Main().IsGray());
 
-    EXPECT_LE(compressed.size(), 7000);
+    EXPECT_LE(compressed.size(), 7000u);
     EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                   /*distmap=*/nullptr, pool),
               1.7777777);
@@ -699,7 +702,7 @@ TEST(JxlTest, RoundtripGrayscale) {
     EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
     EXPECT_TRUE(io2.Main().IsGray());
 
-    EXPECT_LE(compressed.size(), 1300);
+    EXPECT_LE(compressed.size(), 1300u);
     EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                   /*distmap=*/nullptr, pool),
               9.0);
@@ -713,7 +716,7 @@ TEST(JxlTest, RoundtripAlpha) {
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
   ASSERT_TRUE(io.metadata.m.HasAlpha());
   ASSERT_TRUE(io.Main().HasAlpha());
   io.ShrinkTo(300, 300);
@@ -722,9 +725,9 @@ TEST(JxlTest, RoundtripAlpha) {
   cparams.butteraugli_distance = 1.0;
   DecompressParams dparams;
 
-  EXPECT_EQ(8, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(8u, io.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
   EXPECT_TRUE(io.metadata.m.color_encoding.tf.IsSRGB());
   PassesEncoderState enc_state;
   AuxOut* aux_out = nullptr;
@@ -733,7 +736,7 @@ TEST(JxlTest, RoundtripAlpha) {
   CodecInOut io2;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
-  EXPECT_LE(compressed.size(), 10077);
+  EXPECT_LE(compressed.size(), 10077u);
 
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
@@ -748,7 +751,7 @@ TEST(JxlTest, RoundtripAlphaPremultiplied) {
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io_nopremul, pool));
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
   ASSERT_TRUE(io.metadata.m.HasAlpha());
   ASSERT_TRUE(io.Main().HasAlpha());
   io.ShrinkTo(300, 300);
@@ -767,7 +770,7 @@ TEST(JxlTest, RoundtripAlphaPremultiplied) {
   CodecInOut io2;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
-  EXPECT_LE(compressed.size(), 10000);
+  EXPECT_LE(compressed.size(), 10000u);
 
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
@@ -785,7 +788,7 @@ TEST(JxlTest, RoundtripAlphaResampling) {
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
   ASSERT_TRUE(io.metadata.m.HasAlpha());
   ASSERT_TRUE(io.Main().HasAlpha());
 
@@ -802,7 +805,7 @@ TEST(JxlTest, RoundtripAlphaResampling) {
   CodecInOut io2;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
-  EXPECT_LE(compressed.size(), 15000);
+  EXPECT_LE(compressed.size(), 15000u);
 
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
@@ -816,7 +819,7 @@ TEST(JxlTest, RoundtripAlphaResamplingOnlyAlpha) {
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
   ASSERT_TRUE(io.metadata.m.HasAlpha());
   ASSERT_TRUE(io.Main().HasAlpha());
 
@@ -832,7 +835,7 @@ TEST(JxlTest, RoundtripAlphaResamplingOnlyAlpha) {
   CodecInOut io2;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
-  EXPECT_LE(compressed.size(), 31000);
+  EXPECT_LE(compressed.size(), 31000u);
 
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
@@ -846,7 +849,7 @@ TEST(JxlTest, RoundtripAlphaNonMultipleOf8) {
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
   ASSERT_TRUE(io.metadata.m.HasAlpha());
   ASSERT_TRUE(io.Main().HasAlpha());
   io.ShrinkTo(12, 12);
@@ -855,9 +858,9 @@ TEST(JxlTest, RoundtripAlphaNonMultipleOf8) {
   cparams.butteraugli_distance = 1.0;
   DecompressParams dparams;
 
-  EXPECT_EQ(8, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(8u, io.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
   EXPECT_TRUE(io.metadata.m.color_encoding.tf.IsSRGB());
   PassesEncoderState enc_state;
   AuxOut* aux_out = nullptr;
@@ -866,7 +869,7 @@ TEST(JxlTest, RoundtripAlphaNonMultipleOf8) {
   CodecInOut io2;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
-  EXPECT_LE(compressed.size(), 200);
+  EXPECT_LE(compressed.size(), 200u);
 
   // TODO(robryk): Fix the following line in presence of different alpha_bits in
   // the two contexts.
@@ -903,7 +906,7 @@ TEST(JxlTest, RoundtripAlpha16) {
 
   // The image is wider than 512 pixels to ensure multiple groups are tested.
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
   ASSERT_TRUE(io.metadata.m.HasAlpha());
   ASSERT_TRUE(io.Main().HasAlpha());
 
@@ -948,7 +951,7 @@ TEST(JxlTest, JXL_SLOW_TEST(RoundtripLossless8)) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000u);
   // If this test fails with a very close to 0.0 but not exactly 0.0 butteraugli
   // distance, then there is likely a floating point issue, that could be
   // happening either in io or io2. The values of io are generated by
@@ -979,7 +982,7 @@ TEST(JxlTest, JXL_SLOW_TEST(RoundtripLosslessNoEncoderFastPathWP)) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000u);
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, &pool));
 }
@@ -998,7 +1001,7 @@ TEST(JxlTest, JXL_SLOW_TEST(RoundtripLosslessNoEncoderFastPathGradient)) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000u);
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, &pool));
 }
@@ -1017,11 +1020,11 @@ TEST(JxlTest, JXL_SLOW_TEST(RoundtripLosslessNoEncoderVeryFastPathGradient)) {
   DecompressParams dparams;
 
   CodecInOut io2, io3;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000u);
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, &pool));
   cparams.options.skip_encoder_fast_path = false;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 3500000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 3500000u);
   EXPECT_EQ(0.0, ButteraugliDistance(io, io3, cparams.ba_params,
                                      /*distmap=*/nullptr, &pool));
 }
@@ -1038,7 +1041,7 @@ TEST(JxlTest, JXL_SLOW_TEST(RoundtripLossless8Falcon)) {
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 3500000u);
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, &pool));
 }
@@ -1049,24 +1052,24 @@ TEST(JxlTest, RoundtripLossless8Alpha) {
       ReadTestData("wesaturate/500px/tmshre_riaphotographs_alpha.png");
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
-  EXPECT_EQ(8, io.metadata.m.GetAlphaBits());
-  EXPECT_EQ(8, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(8u, io.metadata.m.GetAlphaBits());
+  EXPECT_EQ(8u, io.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
 
   CompressParams cparams = CParamsForLossless();
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 350000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 350000u);
   // If fails, see note about floating point in RoundtripLossless8.
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, pool));
   EXPECT_TRUE(SamePixels(*io.Main().alpha(), *io2.Main().alpha()));
-  EXPECT_EQ(8, io2.metadata.m.GetAlphaBits());
-  EXPECT_EQ(8, io2.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(8u, io2.metadata.m.GetAlphaBits());
+  EXPECT_EQ(8u, io2.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io2.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io2.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io2.metadata.m.bit_depth.exponent_bits_per_sample);
 }
 
 TEST(JxlTest, RoundtripLossless16Alpha) {
@@ -1093,16 +1096,16 @@ TEST(JxlTest, RoundtripLossless16Alpha) {
   io.SetFromImage(std::move(color), io.metadata.m.color_encoding);
   io.Main().SetAlpha(std::move(alpha), /*alpha_is_premultiplied=*/false);
 
-  EXPECT_EQ(16, io.metadata.m.GetAlphaBits());
-  EXPECT_EQ(16, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(16u, io.metadata.m.GetAlphaBits());
+  EXPECT_EQ(16u, io.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
 
   CompressParams cparams = CParamsForLossless();
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 7100);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 7100u);
   // If this test fails with a very close to 0.0 but not exactly 0.0 butteraugli
   // distance, then there is likely a floating point issue, that could be
   // happening either in io or io2. The values of io are generated by
@@ -1118,10 +1121,10 @@ TEST(JxlTest, RoundtripLossless16Alpha) {
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, pool));
   EXPECT_TRUE(SamePixels(*io.Main().alpha(), *io2.Main().alpha()));
-  EXPECT_EQ(16, io2.metadata.m.GetAlphaBits());
-  EXPECT_EQ(16, io2.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(16u, io2.metadata.m.GetAlphaBits());
+  EXPECT_EQ(16u, io2.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io2.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io2.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io2.metadata.m.bit_depth.exponent_bits_per_sample);
 }
 
 TEST(JxlTest, RoundtripLossless16AlphaNotMisdetectedAs8Bit) {
@@ -1150,20 +1153,20 @@ TEST(JxlTest, RoundtripLossless16AlphaNotMisdetectedAs8Bit) {
   io.SetFromImage(std::move(color), io.metadata.m.color_encoding);
   io.Main().SetAlpha(std::move(alpha), /*alpha_is_premultiplied=*/false);
 
-  EXPECT_EQ(16, io.metadata.m.GetAlphaBits());
-  EXPECT_EQ(16, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(16u, io.metadata.m.GetAlphaBits());
+  EXPECT_EQ(16u, io.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
 
   CompressParams cparams = CParamsForLossless();
   DecompressParams dparams;
 
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 3100);
-  EXPECT_EQ(16, io2.metadata.m.GetAlphaBits());
-  EXPECT_EQ(16, io2.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 3100u);
+  EXPECT_EQ(16u, io2.metadata.m.GetAlphaBits());
+  EXPECT_EQ(16u, io2.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io2.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io2.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io2.metadata.m.bit_depth.exponent_bits_per_sample);
   // If fails, see note about floating point in RoundtripLossless8.
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, pool));
@@ -1179,7 +1182,7 @@ TEST(JxlTest, RoundtripYCbCr420) {
   const PaddedBytes yuv420 =
       ReadTestData("imagecompression.info/flower_foveon.png.ffmpeg.y4m");
   CodecInOut io2;
-  ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(yuv420), &io2, pool));
+  ASSERT_TRUE(test::DecodeImageY4M(Span<const uint8_t>(yuv420), &io2));
 
   CompressParams cparams = CParamsForLossless();
   cparams.speed_tier = SpeedTier::kThunder;
@@ -1193,12 +1196,12 @@ TEST(JxlTest, RoundtripYCbCr420) {
   CodecInOut io3;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io3, pool));
 
-  EXPECT_LE(compressed.size(), 1320000);
+  EXPECT_LE(compressed.size(), 1320000u);
 
   // we're comparing an original PNG with a YCbCr 4:2:0 version
   EXPECT_LE(ButteraugliDistance(io, io3, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
-            2.5);
+            2.8);
 }
 
 TEST(JxlTest, RoundtripDots) {
@@ -1208,7 +1211,7 @@ TEST(JxlTest, RoundtripDots) {
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
 
   CompressParams cparams;
   cparams.dots = Override::kOn;
@@ -1216,8 +1219,8 @@ TEST(JxlTest, RoundtripDots) {
   cparams.speed_tier = SpeedTier::kSquirrel;
   DecompressParams dparams;
 
-  EXPECT_EQ(8, io.metadata.m.bit_depth.bits_per_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(8u, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
   EXPECT_TRUE(io.metadata.m.color_encoding.tf.IsSRGB());
   PassesEncoderState enc_state;
@@ -1227,7 +1230,7 @@ TEST(JxlTest, RoundtripDots) {
   CodecInOut io2;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
-  EXPECT_LE(compressed.size(), 400000);
+  EXPECT_LE(compressed.size(), 400000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             2.2);
@@ -1240,15 +1243,15 @@ TEST(JxlTest, RoundtripNoise) {
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
 
-  ASSERT_NE(io.xsize(), 0);
+  ASSERT_NE(io.xsize(), 0u);
 
   CompressParams cparams;
   cparams.noise = Override::kOn;
   cparams.speed_tier = SpeedTier::kSquirrel;
   DecompressParams dparams;
 
-  EXPECT_EQ(8, io.metadata.m.bit_depth.bits_per_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(8u, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
   EXPECT_TRUE(io.metadata.m.color_encoding.tf.IsSRGB());
   PassesEncoderState enc_state;
@@ -1258,7 +1261,7 @@ TEST(JxlTest, RoundtripNoise) {
   CodecInOut io2;
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
-  EXPECT_LE(compressed.size(), 40000);
+  EXPECT_LE(compressed.size(), 40000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, pool),
             2.2);
@@ -1275,18 +1278,18 @@ TEST(JxlTest, RoundtripLossless8Gray) {
   DecompressParams dparams;
 
   EXPECT_TRUE(io.Main().IsGray());
-  EXPECT_EQ(8, io.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(8u, io.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io.metadata.m.bit_depth.exponent_bits_per_sample);
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 130000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 130000u);
   // If fails, see note about floating point in RoundtripLossless8.
   EXPECT_EQ(0.0, ButteraugliDistance(io, io2, cparams.ba_params,
                                      /*distmap=*/nullptr, pool));
   EXPECT_TRUE(io2.Main().IsGray());
-  EXPECT_EQ(8, io2.metadata.m.bit_depth.bits_per_sample);
+  EXPECT_EQ(8u, io2.metadata.m.bit_depth.bits_per_sample);
   EXPECT_FALSE(io2.metadata.m.bit_depth.floating_point_sample);
-  EXPECT_EQ(0, io2.metadata.m.bit_depth.exponent_bits_per_sample);
+  EXPECT_EQ(0u, io2.metadata.m.bit_depth.exponent_bits_per_sample);
 }
 
 #if JPEGXL_ENABLE_GIF
@@ -1296,12 +1299,12 @@ TEST(JxlTest, RoundtripAnimation) {
   const PaddedBytes orig = ReadTestData("jxl/traffic_light.gif");
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
-  ASSERT_EQ(4, io.frames.size());
+  ASSERT_EQ(4u, io.frames.size());
 
   CompressParams cparams;
   DecompressParams dparams;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 3000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 3000u);
 
   EXPECT_EQ(io2.frames.size(), io.frames.size());
   test::CoalesceGIFAnimationWithAlpha(&io);
@@ -1319,12 +1322,12 @@ TEST(JxlTest, RoundtripLosslessAnimation) {
   const PaddedBytes orig = ReadTestData("jxl/traffic_light.gif");
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
-  ASSERT_EQ(4, io.frames.size());
+  ASSERT_EQ(4u, io.frames.size());
 
   CompressParams cparams = CParamsForLossless();
   DecompressParams dparams;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 1200);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 1200u);
 
   EXPECT_EQ(io2.frames.size(), io.frames.size());
   test::CoalesceGIFAnimationWithAlpha(&io);
@@ -1357,9 +1360,7 @@ jxl::Status DecompressJxlToJPEGForTest(
     return JXL_FAILURE("Failed to decode JXL to JPEG");
   }
   io.jpeg_quality = 95;
-  if (!EncodeImageJPG(&io, jxl::JpegEncoder::kLibJpeg, io.jpeg_quality,
-                      jxl::YCbCrChromaSubsampling(), pool, output,
-                      jxl::DecodeTarget::kQuantizedCoeffs)) {
+  if (!extras::EncodeImageJPGCoefficients(&io, output)) {
     return JXL_FAILURE("Failed to generate JPEG");
   }
   return true;
@@ -1413,7 +1414,7 @@ TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression444)) {
   const PaddedBytes orig =
       ReadTestData("imagecompression.info/flower_foveon.png.im_q85_444.jpg");
   // JPEG size is 326'916 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 256000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 256000u);
 }
 
 TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompressionToPixels)) {
@@ -1517,7 +1518,7 @@ TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompressionGray)) {
   const PaddedBytes orig =
       ReadTestData("imagecompression.info/flower_foveon.png.im_q85_gray.jpg");
   // JPEG size is 167'025 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 140000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 140000u);
 }
 
 TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression420)) {
@@ -1525,7 +1526,7 @@ TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression420)) {
   const PaddedBytes orig =
       ReadTestData("imagecompression.info/flower_foveon.png.im_q85_420.jpg");
   // JPEG size is 226'018 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 181050);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 181050u);
 }
 
 TEST(JxlTest,
@@ -1534,7 +1535,7 @@ TEST(JxlTest,
   const PaddedBytes orig = ReadTestData(
       "imagecompression.info/flower_foveon.png.im_q85_luma_subsample.jpg");
   // JPEG size is 216'069 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 181000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 181000u);
 }
 
 TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression444_12)) {
@@ -1543,7 +1544,7 @@ TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression444_12)) {
   const PaddedBytes orig = ReadTestData(
       "imagecompression.info/flower_foveon.png.im_q85_444_1x2.jpg");
   // JPEG size is 329'942 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 256000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 256000u);
 }
 
 TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression422)) {
@@ -1551,7 +1552,7 @@ TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression422)) {
   const PaddedBytes orig =
       ReadTestData("imagecompression.info/flower_foveon.png.im_q85_422.jpg");
   // JPEG size is 265'590 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 209000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 209000u);
 }
 
 TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression440)) {
@@ -1559,7 +1560,7 @@ TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression440)) {
   const PaddedBytes orig =
       ReadTestData("imagecompression.info/flower_foveon.png.im_q85_440.jpg");
   // JPEG size is 262'249 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 209000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 209000u);
 }
 
 TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression_asymmetric)) {
@@ -1569,14 +1570,14 @@ TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression_asymmetric)) {
   const PaddedBytes orig = ReadTestData(
       "imagecompression.info/flower_foveon.png.im_q85_asymmetric.jpg");
   // JPEG size is 262'249 bytes.
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 209000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 209000u);
 }
 
 TEST(JxlTest, JXL_TRANSCODE_JPEG_TEST(RoundtripJpegRecompression420Progr)) {
   ThreadPoolInternal pool(8);
   const PaddedBytes orig = ReadTestData(
       "imagecompression.info/flower_foveon.png.im_q85_420_progr.jpg");
-  EXPECT_LE(RoundtripJpeg(orig, &pool), 181000);
+  EXPECT_LE(RoundtripJpeg(orig, &pool), 181000u);
 }
 
 #endif  // JPEGXL_ENABLE_JPEG
@@ -1597,7 +1598,7 @@ TEST(JxlTest, RoundtripProgressive) {
   cparams.responsive = true;
   cparams.progressive_mode = true;
   CodecInOut io2;
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 40000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 40000u);
   EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
                                 /*distmap=*/nullptr, &pool),
             4.0f);
@@ -1608,14 +1609,14 @@ TEST(JxlTest, RoundtripAnimationPatches) {
   const PaddedBytes orig = ReadTestData("jxl/animation_patches.gif");
   CodecInOut io;
   ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
-  ASSERT_EQ(2, io.frames.size());
+  ASSERT_EQ(2u, io.frames.size());
 
   CompressParams cparams;
   cparams.patches = Override::kOn;
   DecompressParams dparams;
   CodecInOut io2;
   // 40k with no patches, 27k with patch frames encoded multiple times.
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 24000);
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 24000u);
 
   EXPECT_EQ(io2.frames.size(), io.frames.size());
   // >10 with broken patches
